!=======================================================================
!                   _____________________________
!               \\//-----------------------------
!                )(   //=  //= //= //// //)) //
!               //\\ //_  //  //  /__/ //\\ //
!=======================================================================
!                  XEFFORT LIBRARY SAMPLES
!=======================================================================
! XDblBuffer2.f90 - Usage of double-buffering with help of XFTGDI
! module in a Win32 application
!
! 2005. Jugoslav Dujic (jdujic@uns.ns.ac.yu)
! 
! You are free to use the code for both commercial and non-commercial
! use. The code is provided as-is without warranties. You are free to
! distribute and modify this source code provided that the list of 
! original author(s) remains untouched and your revisions are indicated
! as such. Contributions are welcome.
!=======================================================================
!  FUNCTIONS:
!   WinMain()      - Entry point for the application;
!                    displays the main window; processes the message loop
!   MainWndProc()  - processes messages for the main window
!   CenterWindow() - centers one window over another
!   AboutDlgProc() - processes messages for the about box
!

!****************************************************************************
!
!  FUNCTION: WinMain( hInstance, hPrevInstance, lpszCmdLine, nCmdShow )
!
!  PURPOSE:  Entry point for the application
!
!  COMMENTS: Displays the main window and processes the message loop
!
!****************************************************************************

integer*4 function WinMain( hInstance, hPrevInstance, lpszCmdLine, nCmdShow )
!DEC$ IF DEFINED(_X86_)
!DEC$ ATTRIBUTES STDCALL, ALIAS : '_WinMain@16' :: WinMain
!DEC$ ELSE
!DEC$ ATTRIBUTES STDCALL, ALIAS : 'WinMain' :: WinMain
!DEC$ ENDIF

    use user32
    use kernel32
    use XDblBuffer2Globals

    implicit none

    integer*4 hInstance
    integer*4 hPrevInstance
    integer*4 lpszCmdLine
    integer*4 nCmdShow

    include 'XDblBuffer2.fi'
    include 'resource.fd'

    ! Variables
    type (T_WNDCLASS)       wc
    type (T_MSG)            mesg
    integer*4               ret
    logical*4               lret
    integer                 haccel

    character(SIZEOFAPPNAME) lpszClassName
    character(SIZEOFAPPNAME) lpszIconName
    character(SIZEOFAPPNAME) lpszAppName
    character(SIZEOFAPPNAME) lpszMenuName
    character(SIZEOFAPPNAME) lpszAccelName

    ghInstance = hInstance
    ghModule = GetModuleHandle(NULL)
    ghwndMain = NULL

    lpszClassName ="XDblBuffer2"C
    lpszAppName ="XDblBuffer2"C
    lpszIconName ="XDBLBUFFER2"C  
    lpszMenuName ="XDblBuffer2"C
    lpszAccelName ="XDblBuffer2"C

    !  If this is the first instance of the application, register the
    !  window class(es)
    if (hPrevInstance .eq. 0) then
        !  Main window
         wc%lpszClassName = LOC(lpszClassName)
         wc%lpfnWndProc = LOC(MainWndProc)
         !=== JD's revision ===========
         !When using double-buffering, we want control over redrawing (handle WM_SIZE)
         wc%style = IOR(CS_VREDRAW , CS_HREDRAW)
         wc%style = 0
         !=== /JD's revision ===========
         wc%hInstance = hInstance
         wc%hIcon = LoadIcon( hInstance, LOC(lpszIconName))
         wc%hCursor = LoadCursor( NULL, IDC_ARROW )
         !=== JD's revision ===========
         !Also, we erase backrgound in Redraw(), so default erasing will cause flicker
         wc%hbrBackground = ( COLOR_WINDOW+1 )
         !=== /JD's revision ===========

         wc%lpszMenuName = NULL
         wc%cbClsExtra = 0
         wc%cbWndExtra = 0
         if (RegisterClass(wc) == 0) goto 99999
    end if

    ! Load the window's menu and accelerators and create the window
    !
    ghMenu = LoadMenu(hInstance, LOC(lpszMenuName))
    if (ghMenu == 0) goto 99999
    haccel = LoadAccelerators(hInstance, LOC(lpszAccelName))
    if (haccel == 0) goto 99999

    ghwndMain = CreateWindowEx(  0, lpszClassName,                 &
                                 lpszAppName,                      &
                                 !=== JD's revision ===========
                                 INT(WS_OVERLAPPEDWINDOW + WS_VSCROLL + WS_HSCROLL),         &
                                 !=== end JD's revision =======
                                 CW_USEDEFAULT,                    &
                                 0,                                &
                                 CW_USEDEFAULT,                    &
                                 0,                                &
                                 NULL,                             &
                                 ghMenu,                           &
                                 hInstance,                        &
                                 NULL                              &
                              )
    if (ghwndMain == 0) goto 99999

    !=== JD's revision ===========
    xMemDC = XMemoryDC(IMAGE_WIDTH, IMAGE_HEIGHT)
    maxVertex = 7
    ret = CheckMenuItem(ghMenu, IDM_7, MF_BYCOMMAND+MF_CHECKED)
    call Redraw(xMemDC)
    !=== /JD's revision ===========
    lret = ShowWindow( ghwndMain, nCmdShow )

    ! Read and process messsages
    do while( GetMessage (mesg, NULL, 0, 0) ) 
       if ( TranslateAccelerator (mesg%hwnd, haccel, mesg) == 0) then
           lret = TranslateMessage( mesg )
           ret  = DispatchMessage( mesg )
       end if
    end do

    !=== JD's revision ===========
    lret = XDeleteDC(xMemDC)
    !=== /JD's revision ===========
    WinMain = mesg.wParam
    return

99999 &

    ret = MessageBox(ghwndMain, "Error initializing application XDblBuffer2"C, &
                     "Error"C, MB_OK)
    WinMain = 0

end 

!****************************************************************************
!
!  FUNCTION: MainWndProc ( hWnd, mesg, wParam, lParam )
!
!  PURPOSE:  Processes messages for the main window
!
!  COMMENTS:
!
!****************************************************************************

integer function MainWndProc ( hWnd, mesg, wParam, lParam )
!DEC$ IF DEFINED(_X86_)
!DEC$ ATTRIBUTES STDCALL, ALIAS : '_MainWndProc@16' :: MainWndProc
!DEC$ ELSE
!DEC$ ATTRIBUTES STDCALL, ALIAS : 'MainWndProc' :: MainWndProc
!DEC$ ENDIF

    use dfwin
    use XDblBuffer2Globals

    implicit none

    integer*4 hWnd
    integer*4 mesg
    integer*4 wParam
    integer*4 lParam
    !=== JD's revision ===========
    type(X_DC)::  xDC  !this is the real window's DC (canvas)
    type(T_PAINTSTRUCT)::  PS
    type(T_SCROLLINFO)::   SI
    integer::              iX, iY, iFlag
    !=== /JD's revision ===========

    include 'resource.fd'

    interface 
    integer*4 function  AboutDlgProc( hwnd, mesg, wParam, lParam )
    !DEC$ IF DEFINED(_X86_)
    !DEC$ ATTRIBUTES STDCALL, ALIAS : '_AboutDlgProc@16' :: AboutDlgProc
    !DEC$ ELSE
    !DEC$ ATTRIBUTES STDCALL, ALIAS : 'AboutDlgProc' :: AboutDlgProc
    !DEC$ ENDIF

    integer*4 hwnd 
    integer*4 mesg  
    integer*4 wParam
    integer*4 lParam 
    end function 
    end interface

    ! Variables
    integer*4           ret
    character(SIZEOFAPPNAME)  lpszName, lpszHelpFileName, lpszContents, lpszMessage
    character(SIZEOFAPPNAME)  lpszHeader

    select case ( mesg )

    ! WM_DESTROY: PostQuitMessage() is called 
      case (WM_DESTROY)
          call PostQuitMessage( 0 )
          MainWndProc = 0
          return

    ! WM_COMMAND: user command
      case (WM_COMMAND)
        select case ( IAND(wParam, 16#ffff ) )
 
            case (IDM_EXIT)
                ret = SendMessage( hWnd, WM_CLOSE, 0, 0 )
                MainWndProc = 0
                return
  
            case (IDM_ABOUT)
                lpszName = "AboutDlg"C
                ret = DialogBoxParam(ghInstance,LOC(lpszName),hWnd,& 
                  LOC(AboutDlgProc), 0)
                MainWndProc = 0
                return

            case (IDM_HELPCONTENTS)  
                lpszHelpFileName ="\\XDblBuffer2.hlp"C
                lpszContents = "CONTENTS"C
                if (WinHelp (hWnd, lpszHelpFileName, HELP_KEY, &
                               LOC(lpszContents)) .EQV. .FALSE.) then
                lpszMessage = "Unable to activate help"C
                lpszHeader = "XDblBuffer2"
                ret = MessageBox (hWnd,                             &
                                 lpszMessage,                       &
                                 lpszHeader,                        &
                                 IOR(MB_SYSTEMMODAL,                &
                                 IOR(MB_OK, MB_ICONHAND)))
                end if
                MainWndProc = 0
                return

            case (IDM_HELPSEARCH)
                lpszHelpFileName ="\\XDblBuffer2.hlp"C
                lpszContents = "CONTENTS"C
                if (WinHelp(hWnd, "XDblBuffer2.hlp"C,            &
                       HELP_PARTIALKEY, LOC(""C)) .EQV. .FALSE.) then
                   lpszMessage = "Unable to activate help"C
                   lpszHeader = "XDblBuffer2"C
                   ret = MessageBox (hWnd,                          &
                                 lpszMessage,                       &
                                 lpszHeader,                        &
                                 IOR(MB_SYSTEMMODAL ,               &
                                 IOR(MB_OK, MB_ICONHAND)))
                end if
                MainWndProc = 0
                return

            case (IDM_HELPHELP)
                if (WinHelp(hWnd, ""C, HELP_HELPONHELP, 0).EQV. .FALSE.)& 
                                                       then
                   lpszMessage = "Unable to activate help"C
                   lpszHeader = "XDblBuffer2"C
                   ret = MessageBox (GetFocus(),                    &
                                 lpszMessage,                       &
                                 lpszHeader,                        &
                                 IOR(MB_SYSTEMMODAL,IOR(MB_OK, MB_ICONHAND)))
                end if
                MainWndProc = 0
                return

            ! All of the other possible menu options are currently disabled
            !=== JD's revision ===========
            !Handle menu items
            case (IDM_5:IDM_15)
               select case(maxVertex)
               case(5)
                  ret = CheckMenuItem(ghMenu, IDM_5, MF_BYCOMMAND+MF_UNCHECKED)
               case(7)
                  ret = CheckMenuItem(ghMenu, IDM_7, MF_BYCOMMAND+MF_UNCHECKED)
               case(9)
                  ret = CheckMenuItem(ghMenu, IDM_9, MF_BYCOMMAND+MF_UNCHECKED)
               case(11)
                  ret = CheckMenuItem(ghMenu, IDM_11, MF_BYCOMMAND+MF_UNCHECKED)
               case(13)
                  ret = CheckMenuItem(ghMenu, IDM_13, MF_BYCOMMAND+MF_UNCHECKED)
               case(15)
                  ret = CheckMenuItem(ghMenu, IDM_15, MF_BYCOMMAND+MF_UNCHECKED)
               end select

               select case(IAND(wParam, 16#ffff))
               case(IDM_5)
                  maxVertex = 5
               case(IDM_7)
                  maxVertex = 7
               case(IDM_9)
                  maxVertex = 9
               case(IDM_11)
                  maxVertex = 11
               case(IDM_13)
                  maxVertex = 13
               case(IDM_15)
                  maxVertex = 15
               end select

               call Redraw(xMemDC)
               ret = CheckMenuItem(ghMenu, IAND(wParam, 16#ffff), MF_BYCOMMAND+MF_CHECKED)

            !=== JD's revision ===========
            case DEFAULT
                MainWndProc = DefWindowProc( hWnd, mesg, wParam, lParam )
                return
        end select
      !=== JD's revision ===========
      case (WM_PAINT)
         !WM_PAINT is sent whenever any portion of the window has to be updated
         xDC%hDC = BeginPaint(hWnd, PS)
         !Copy the contents of offscreen DC (only the required region, described by
         !PS%rcPaint is copied; it could be entire window as well, slightly less efficient):
         iX = GetScrollPos(hWnd, SB_HORZ)
         iY = GetScrollPos(hWnd, SB_VERT)
         ret = XBitCopy(xMemDC, PS%rcPaint%Left+iX, PS%rcPaint%Top+iY, PS%rcPaint%Right+iX, PS%rcPaint%Bottom+iY, &
               xDC, PS%rcPaint%Left, PS%rcPaint%Top)
         ret = EndPaint(hWnd, PS)
         MainWndProc = 0
      case (WM_VSCROLL, WM_HSCROLL)
          !sent whenever user moves the scrollbar
         if (mesg == WM_VSCROLL) then
            iFlag = SB_VERT
         else
            iFlag = SB_HORZ
         end if
         si%Size = SIZEOF(si)
         si%Mask = SIF_ALL
         ret = GetScrollInfo(hWnd, iFlag, si)
         select case (LOWORD(wParam))
         case (SB_TOP)
            si%Pos = si%Min
         case (SB_BOTTOM)
            si%Pos = si%Max
         case (SB_LINEUP)
            si%Pos = si%Pos - 10
         case (SB_LINEDOWN)
            si%Pos = si%Pos + 10
         case (SB_PAGEUP)
            si%Pos = si%Pos - si%Page
         case (SB_PAGEDOWN)
            si%Pos = si%Pos + si%Page
         case (SB_THUMBTRACK)
            si%Pos = si%trackpos
         end select
         ret = SetScrollPos(hWnd, iFlag, si%Pos ,.true.)

          call XUpdateWindow(hWnd)
          MainWndProc = DefWindowProc( hWnd, mesg, wParam, lParam )

      case (WM_SIZE)
          si%Size = SIZEOF(si)
          si%Min = 0
          si%Max = IMAGE_WIDTH
          si%Page = LOWORD(lParam)  ! width of client area 
          si%Mask = SIF_PAGE .or. SIF_RANGE
          ret = SetScrollInfo(hWnd, SB_HORZ, si, .false.)
          si%Min = 0
          si%Max = IMAGE_HEIGHT
          si%Page = HIWORD(lParam)  ! height of client area 
          si%Mask = SIF_PAGE .or. SIF_RANGE
          ret = SetScrollInfo(hWnd, SB_VERT, si, .true.)

          call XUpdateWindow(hWnd)
          MainWndProc = DefWindowProc( hWnd, mesg, wParam, lParam )
      !=== /JD's revision ===========

    ! Let the default window proc handle all other messages
      case default
          MainWndProc = DefWindowProc( hWnd, mesg, wParam, lParam )

    end select

end function MainWndProc
!=== JD's revision ===========
!****************************************************************************
!
!  FUNCTION: Redraw ()
!
!  PURPOSE:  Draws the desired image onto off-screen xDC; at the end,
!            calls XUpdateWindow, which triggers WM_PAINT, so that the image
!            is copied to the window
!
subroutine Redraw(xDC)

use dfwin
use XDblBuffer2Globals

implicit none

type(X_DC)::  xDC

integer::      i, idim, ixc, iyc, n
real::         xc, yc
logical::      b
type(t_rect):: rc

!Window's client area
b = GetClientRect(ghwndMain, rc)

!Erase the background
call XSetViewport(xDC, 0, 0, IMAGE_WIDTH, IMAGE_HEIGHT)
call XSetBrush(xDC, 0)
call XSetPen(xDC, 0, PS_NULL)
b = XRectangle(xDC, 0, 0, IMAGE_WIDTH, IMAGE_HEIGHT)

call XSetWindow(xDC, -50., -50., 50., 50.)

!Draw the rosette
call XSetPen(xDC, XCOLOR_WHITE, iWidth = 3)

call XMoveTo_W(xDC, 50., 0.)
n = 0
do i = 1, maxVertex+1
   xc = 50. * cosd(n * 360./maxVertex)
   yc = 50. * sind(n * 360./maxVertex)
   b = XLineTo_W(xDC, xc, yc)
   n = n + maxVertex/2
   n = mod(n, maxVertex)
end do

!Refresh the contents
call XUpdateWindow(ghwndMain)

end subroutine Redraw
!=== /JD's revision ===========

!****************************************************************************
!
!  FUNCTION: CenterWindow (HWND, HWND)
!
!  PURPOSE:  Center one window over another
!
!  COMMENTS: Dialog boxes take on the screen position that they were designed
!            at, which is not always appropriate. Centering the dialog over a
!            particular window usually results in a better position.
!
!****************************************************************************

subroutine CenterWindow (hwndChild, hwndParent)

    use user32
    use gdi32 
    use XDblBuffer2Globals

    implicit none

    integer         hwndChild, hwndParent

    include 'resource.fd'

    ! Variables
    type (T_RECT)   rChild, rParent
    integer         wChild, hChild, wParent, hParent
    integer         wScreen, hScreen, xNew, yNew
    integer         hdc
    integer*4       retval

    ! Get the Height and Width of the child window
       retval = GetWindowRect (hwndChild, rChild)
       wChild = rChild.right - rChild.left
       hChild = rChild.bottom - rChild.top

    ! Get the Height and Width of the parent window
       retval = GetWindowRect (hwndParent, rParent)
       wParent = rParent.right - rParent.left
       hParent = rParent.bottom - rParent.top

    ! Get the display limits
       hdc = GetDC (hwndChild)
       wScreen = GetDeviceCaps (hdc, HORZRES)
       hScreen = GetDeviceCaps (hdc, VERTRES)
       retval = ReleaseDC (hwndChild, hdc)

    ! Calculate new X position, then adjust for screen
       xNew = rParent.left + ((wParent - wChild) /2)
       if (xNew .LT. 0) then
          xNew = 0
       else if ((xNew+wChild) .GT. wScreen) then
          xNew = wScreen - wChild
       end if

    ! Calculate new Y position, then adjust for screen
       yNew = rParent.top  + ((hParent - hChild) /2)
       if (yNew .LT. 0) then
          yNew = 0
       else if ((yNew+hChild) .GT. hScreen) then
          yNew = hScreen - hChild
       end if

    ! Set it, and return
       retval = SetWindowPos (hwndChild, NULL, xNew, yNew, 0, 0,      &
                      IOR(SWP_NOSIZE , SWP_NOZORDER))
end  

 
!/****************************************************************************
!
!  FUNCTION: AboutDlgProc(HWND, UINT, WPARAM, LPARAM)
!
!  PURPOSE:  Processes messages for "About" dialog box
!
!  COMMENTS: Display version information from the version section of the
!            application resource.  Wait for user to click on "Ok" button,
!            then close the dialog box.
!
!****************************************************************************/

integer*4 function AboutDlgProc( hDlg, message, uParam, lParam )
!DEC$ IF DEFINED(_X86_)
!DEC$ ATTRIBUTES STDCALL, ALIAS : '_AboutDlgProc@16' :: AboutDlgProc
!DEC$ ELSE
!DEC$ ATTRIBUTES STDCALL, ALIAS : 'AboutDlgProc' :: AboutDlgProc
!DEC$ ENDIF

    use kernel32
    use user32
    use gdi32
    use dfwbase
    use version
    use XDblBuffer2Globals

    implicit none

    integer     hDlg        ! window handle of the dialog box
    integer     message     ! type of message
    integer     uParam      ! message-specific information
    integer     lParam

    include 'resource.fd'

    ! Variables
    integer*4   hfontDlg
    save        hfontDlg

    integer     dwVerHnd
    integer     dwVerInfoSize
    integer     uVersionLen
    integer     bRetCode
    integer     i
    character*256   szFullPath
    character*256   szResult
    character*256   szGetName
    character*256   lpversion

    integer*4   lpstrVffInfo
    integer*4   hMem
    integer*4   ret

    select case (message)
      case (WM_INITDIALOG)   ! message: initialize dialog box
         ! Create a font to use
         hfontDlg = CreateFont(14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,& 
                        IOR(INT(VARIABLE_PITCH) , INT(FF_SWISS)), ""C)

         ! Center the dialog over the application window
         call CenterWindow (hDlg, GetWindow (hDlg, GW_OWNER))

         ! Get version information from the application
         ret = GetModuleFileName (INT(ghInstance), szFullPath,     &
                               len(szFullPath))
         dwVerInfoSize = GetFileVersionInfoSize(szFullPath,   &
                                       LOC(dwVerHnd))

         if (dwVerInfoSize .NE. 0) then
            ! If we were able to get the information, process it:
            hMem = GlobalAlloc(GMEM_MOVEABLE, INT(dwVerInfoSize))
            lpstrVffInfo  = GlobalLock(hMem)
            ret = GetFileVersionInfo (szFullPath, dwVerHnd, &
            dwVerInfoSize, lpstrVffInfo)

            ! Walk through the dialog items that we want to replace:
            do i = IDC_VER1, IDC_VER5
               ret = GetDlgItemText(hDlg, i, szResult,      &     
                             len(szResult))
                
               szGetName = "\\StringFileInfo\\040904E4\\"C               
               ret =lstrcat(szGetName,szResult)

               bRetCode  =  VersionQueryValue(lpstrVffInfo,       &
                                            LOC(szGetName),       &
                                            LOC(lpVersion),       &
                                            LOC(uVersionLen))

               if ( bRetCode .NE. 0 ) then
                  ! Replace dialog item text with version info
                  ret = lstrcpy(szResult,lpVersion)
                  ret = SetDlgItemText(hDlg, i,szResult)
                  ret = SendMessage (GetDlgItem (hDlg, i),   &
                                   WM_SETFONT, hfontDlg, TRUE)
               end if
            end do 

            ret = GlobalUnlock(hMem)
            ret = GlobalFree(hMem)
         end if 
         AboutDlgProc = 1
         return
      case (WM_COMMAND)                      ! message: received a command
         if ((IAND(uParam,16#ffff) .EQ. IDOK) & !OK Selected?
            .OR. (IAND(uParam,16#ffff) .EQ. IDCANCEL)) then ! Close command?
            ret = EndDialog(hDlg, TRUE)      ! Exit the dialog
            ret = DeleteObject (hfontDlg)
            AboutDlgProc = 1
            return
         end if
    end select  
    AboutDlgProc = 0 ! Didn't process the message
    return
end 

